@@ -11,6 +11,9 @@
 
 #include "npy_pycompat.h"
 
+#if defined(HAVE_STRTOLD_L) && !defined(_GNU_SOURCE)
+# define _GNU_SOURCE
+#endif
 #include <locale.h>
 #include <stdio.h>
 
