@@ -88,7 +88,7 @@ _set_madvise_hugepage(PyObject *NPY_UNUSED(self), PyObject *enabled_obj)
  * base function for data cache with 1 byte buckets and dimension cache with
  * sizeof(npy_intp) byte buckets
  */
-static NPY_INLINE void *
+static inline void *
 _npy_alloc_cache(npy_uintp nelem, npy_uintp esz, npy_uint msz,
                  cache_bucket * cache, void * (*alloc)(size_t))
 {
@@ -126,7 +126,7 @@ _npy_alloc_cache(npy_uintp nelem, npy_uintp esz, npy_uint msz,
  * return pointer p to cache, nelem is number of elements of the cache bucket
  * size (1 or sizeof(npy_intp)) of the block pointed too
  */
-static NPY_INLINE void
+static inline void
 _npy_free_cache(void * p, npy_uintp nelem, npy_uint msz,
                 cache_bucket * cache, void (*dealloc)(void *))
 {
@@ -206,7 +206,7 @@ npy_free_cache_dim(void * p, npy_uintp sz)
 }
 
 /* Similar to array_dealloc in arrayobject.c */
-static NPY_INLINE void
+static inline void
 WARN_NO_RETURN(PyObject* warning, const char * msg) {
     if (PyErr_WarnEx(warning, msg, 1) < 0) {
         PyObject * s;
@@ -364,7 +364,7 @@ PyDataMem_RENEW(void *ptr, size_t size)
 // The default data mem allocator malloc routine does not make use of a ctx.
 // It should be called only through PyDataMem_UserNEW
 // since itself does not handle eventhook and tracemalloc logic.
-static NPY_INLINE void *
+static inline void *
 default_malloc(void *NPY_UNUSED(ctx), size_t size)
 {
     return _npy_alloc_cache(size, 1, NBUCKETS, datacache, &malloc);
@@ -373,7 +373,7 @@ default_malloc(void *NPY_UNUSED(ctx), size_t size)
 // The default data mem allocator calloc routine does not make use of a ctx.
 // It should be called only through PyDataMem_UserNEW_ZEROED
 // since itself does not handle eventhook and tracemalloc logic.
-static NPY_INLINE void *
+static inline void *
 default_calloc(void *NPY_UNUSED(ctx), size_t nelem, size_t elsize)
 {
     void * p;
@@ -395,7 +395,7 @@ default_calloc(void *NPY_UNUSED(ctx), size_t nelem, size_t elsize)
 // The default data mem allocator realloc routine does not make use of a ctx.
 // It should be called only through PyDataMem_UserRENEW
 // since itself does not handle eventhook and tracemalloc logic.
-static NPY_INLINE void *
+static inline void *
 default_realloc(void *NPY_UNUSED(ctx), void *ptr, size_t new_size)
 {
     return realloc(ptr, new_size);
@@ -404,7 +404,7 @@ default_realloc(void *NPY_UNUSED(ctx), void *ptr, size_t new_size)
 // The default data mem allocator free routine does not make use of a ctx.
 // It should be called only through PyDataMem_UserFREE
 // since itself does not handle eventhook and tracemalloc logic.
-static NPY_INLINE void
+static inline void
 default_free(void *NPY_UNUSED(ctx), void *ptr, size_t size)
 {
     _npy_free_cache(ptr, size, NBUCKETS, datacache, &free);
@@ -424,10 +424,7 @@ PyDataMem_Handler default_handler = {
 };
 /* singleton capsule of the default handler */
 PyObject *PyDataMem_DefaultHandler;
-
-#if (!defined(PYPY_VERSION_NUM) || PYPY_VERSION_NUM >= 0x07030600)
 PyObject *current_handler;
-#endif
 
 int uo_index=0;   /* user_override index */
 
@@ -539,7 +536,6 @@ NPY_NO_EXPORT PyObject *
 PyDataMem_SetHandler(PyObject *handler)
 {
     PyObject *old_handler;
-#if (!defined(PYPY_VERSION_NUM) || PYPY_VERSION_NUM >= 0x07030600)
     PyObject *token;
     if (PyContextVar_Get(current_handler, NULL, &old_handler)) {
         return NULL;
@@ -554,27 +550,6 @@ PyDataMem_SetHandler(PyObject *handler)
     }
     Py_DECREF(token);
     return old_handler;
-#else
-    PyObject *p;
-    p = PyThreadState_GetDict();
-    if (p == NULL) {
-        return NULL;
-    }
-    old_handler = PyDict_GetItemString(p, "current_allocator");
-    if (old_handler == NULL) {
-        old_handler = PyDataMem_DefaultHandler
-    }
-    Py_INCREF(old_handler);
-    if (handler == NULL) {
-        handler = PyDataMem_DefaultHandler;
-    }
-    const int error = PyDict_SetItemString(p, "current_allocator", handler);
-    if (error) {
-        Py_DECREF(old_handler);
-        return NULL;
-    }
-    return old_handler;
-#endif
 }
 
 /*NUMPY_API
@@ -585,28 +560,10 @@ NPY_NO_EXPORT PyObject *
 PyDataMem_GetHandler()
 {
     PyObject *handler;
-#if (!defined(PYPY_VERSION_NUM) || PYPY_VERSION_NUM >= 0x07030600)
     if (PyContextVar_Get(current_handler, NULL, &handler)) {
         return NULL;
     }
     return handler;
-#else
-    PyObject *p = PyThreadState_GetDict();
-    if (p == NULL) {
-        return NULL;
-    }
-    handler = PyDict_GetItem(p, npy_ma_str_current_allocator);
-    if (handler == NULL) {
-        handler = PyCapsule_New(&default_handler, "mem_handler", NULL);
-        if (handler == NULL) {
-            return NULL;
-        }
-    }
-    else {
-        Py_INCREF(handler);
-    }
-    return handler;
-#endif
 }
 
 NPY_NO_EXPORT PyObject *
