@@ -742,12 +742,15 @@ class AccelerateSystemDependency(BLASLAPACKMixin, SystemDependency):
             self.detect(kwargs)
 
     def check_macOS_recent_enough(self) -> bool:
-        # We need the SDK to be >=13.3 (meaning at least XCode 14.3)
-        cmd = ['xcrun', '-sdk', 'macosx', '--show-sdk-version']
-        sdk_version = subprocess.run(cmd, capture_output=True, check=True, text=True).stdout.strip()
         macos_version = platform.mac_ver()[0]
         deploy_target = os.environ.get('MACOSX_DEPLOYMENT_TARGET', macos_version)
-        return sdk_version >= '13.3' and deploy_target >= '13.3'
+        if not mesonlib.version_compare(deploy_target, '>=13.3'):
+            return False
+
+        # We also need the SDK to be >=13.3 (meaning at least XCode 14.3)
+        cmd = ['xcrun', '-sdk', 'macosx', '--show-sdk-version']
+        sdk_version = subprocess.run(cmd, capture_output=True, check=True, text=True).stdout.strip()
+        return mesonlib.version_compare(sdk_version, '>=13.3')
 
     def detect(self, kwargs: T.Dict[str, T.Any]) -> None:
         from .framework import ExtraFrameworkDependency
